/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/***********************************************************************************/
/*                                                                                 */
/* Program:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* eq2hor.c                                                                        */
/*                                                                                 */
/*                                                                                 */
/* Version:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* First version: October 1995                                                     */
/* Latest update: 2003-08-23                                                       */
/*                                                                                 */
/*                                                                                 */
/* Author:                                                                         */
/* =======                                                                         */
/*                                                                                 */
/* Program by: Stefan Spaennare                                                    */
/*                                                                                 */
/*                                                                                 */
/* Purpose:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* The program calculates horizon coordinates (altitude, azimuth and zenith-       */
/* distance) of a celestial object from equatorial coordinates (rigth acension     */
/* and declination) of the celestial object, longitude and latitude of location    */
/* on on earth, epoch, date and Universal Time (UT).                               */
/*                                                                                 */
/*                                                                                 */
/* Algorithms:                                                                     */
/* ===========                                                                     */
/*                                                                                 */
/* The program use a very long time accurate method for correcting right acension  */
/* and declination from year epoch to actual date for the precession of earth.     */
/* The epoch is supposed to start on January 1st the year of the epoch.            */
/* Maximum error due to precession calculation is about 0.2 arcsec.                */
/*                                                                                 */
/* The program does not take into account proper motion of the celestial object    */
/* (from year epoch to actual date) or refraction in the atmosphere of the earth.  */ 
/*                                                                                 */
/*                                                                                 */
/* References:                                                                     */
/* ===========                                                                     */
/*                                                                                 */
/* The book "Practical Astronomy with Your Calculator", Third Edition,             */
/* by Peter Duffett-Smith, 1988.                                                   */
/*                                                                                 */
/*                                                                                 */
/* Compiling:                                                                      */
/* ==========                                                                      */
/*                                                                                 */
/* >gcc -O3 -o eq2hor eq2hor.c -lm                                                 */
/*                                                                                 */
/*                                                                                 */
/* Input data:                                                                     */
/* ===========                                                                     */
/*                                                                                 */
/* >Usage: eq2hor halphai malphai salphai                                          */
/* >              ddeltai mdeltai sdeltar                                          */
/* >              dloi    mloi    slor                                             */
/* >              dlai    mlai    slar                                             */
/* >              epochi                                                           */
/* >              yeari monthi dayi houri mini secr                                */
/*                                                                                 */
/* halphai malphai salphar: Right acension of celestial object (hour min sec).     */
/* ddeltai mdeltai sdeltar: Declination of celestial object (degrees min sec).     */
/* dloi mloi slor: Longitude of location on earth (degrees min sec).               */
/* dlai mlai slar: Latitude of location on earth (degrees min sec).                */
/* epochi: Epoch (year) for equatiorial coordinates of celectial object.           */
/* yeari monthi dayi: Actual date.                                                 */
/* houri mini secr: Universal time (UT) at actual date.                            */
/*                                                                                 */
/* If epochi <= 0 no correction is made for precession.                            */
/*                                                                                 */
/* Input variables ending with "i" are integers and ending with "r" are real.      */
/* Negative values of coordinates are typed with a minus sign (-) on hour          */
/* or degree only (not on min or sec).                                             */
/*                                                                                 */
/*                                                                                 */
/* Example:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* The following input line                                                        */
/*                                                                                 */
/* >eq2hor 9 2 38.2 -68 13 43.7 -70 24 17.28 -24 37 40.44 2000 1999 3 29 1 32 43.0 */
/*                                                                                 */
/* should give the following result                                                */
/*                                                                                 */
/* >Altitude        =  46.36010966 deg =   46 deg 21 min 36.39 sec                 */
/* >                                                                               */
/* >Azimuth         = 181.68655824 deg =  181 deg 41 min 11.61 sec                 */
/* >                                                                               */
/* >Zenith distance =  43.63989034 deg =   43 deg 38 min 23.61 sec                 */
/*                                                                                 */
/*                                                                                 */
/***********************************************************************************/     


#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>


double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


double rad(x)
double x;
{ 
   return(pi*x/180.0);
} /* rad */


double deg(x)
double x;
{
   return(180.0*x/pi);
} /* deg */


double in_24(x)
double x;
{
   double x2;

   x2=x/24.0;
   x2=24.0*(x2-floor(x2));
   if (x2 < 0.0) {
      x2=x2+24.0;
   } /* if */

   return(x2);

} /* in_24 */


double in_2pi(x)
double x;
{
   double x2,pi2;

   pi2=2.0*pi;

   x2=x/pi2;
   x2=pi2*(x2-floor(x2));
   if (x2 < 0.0) {
      x2=x2+pi2;
   } /* if */

   return(x2);

} /* in_2pi */


double dms_deg(degi,mini,secr)
int degi,mini;
double secr;
{
   double temp;

   temp=fabs((double)(degi))+(double)(mini)/60.0+secr/3600.0;

   if (degi >=0) {
      return(temp);
   }
   else {
      return(-temp);
   } /* if */

} /* dms_deg */


void deg_dms(degr,degi,mini,secr)
double degr;
int *degi;
int *mini;
double *secr;
{
    int d,m;

    double s,deg2,temp;

    deg2=fabs(degr);

    d=(int)(deg2);
    temp=deg2-(double)(d);
    m=(int)(60.0*temp);
    s=3600.0*temp-60.0*(double)(m);

    if (degr >= 0.0) {
       *degi=d;
    }
    else {
       *degi=-d;
    } /* if */

    *mini=m;
    *secr=s;

} /* deg_dms */


double julian_day(year,month,day)
int year,month,day;
{
   double f,g,a,jdd;

   if (month>=3) {
      f=(double)(year);
      g=(double)(month);
   }
   else {
      f=(double)(year-1);
      g=(double)(month+12);
   } /* if */

   a=2.0-floor(f/100.0)+floor(f/400.0);

   jdd=floor(365.25*f)+floor(30.6001*(g+1.0))+
       (double)(day)+a+1720994.5;

   return(jdd);

} /* julian_day */


double ut_gst(year,month,day,ut)
int year,month,day;
double ut;
{
   double jd0,s,t,t0,gst;

   jd0=julian_day(year,month,day);
   s=jd0-2451545.0;
   t=s/36525.0;
   t0=6.697374558+2400.051336*t+0.000025862*t*t;
   t0=in_24(t0);
   gst=in_24(t0+1.002737909*ut);

   return(gst);

} /* ut_gst */


void matrix_mult(m3_out,m3_in,m33_in)
double m3_out[5];
double m3_in[5];
double m33_in[5][5];
{
   int i,j,n;
   double sum;

   n=3;

   for (i=1; i<=n; i++) {
      sum=0.0;
      for (j=1; j<=n; j++) {
          sum=sum+m33_in[i][j]*m3_in[j];
      } /* for j */
      m3_out[i]=sum;
   } /* for i */

} /* matrix_mult */ 


void param(zaeta,z,taeta,jd)
double *zaeta,*z,*taeta;
double jd;
{
   double t,t2,t3;
 
   t=(jd-2451545.0)/36525.0;
   t2=t*t; t3=t*t*t;

   *zaeta=0.6406161*t+0.0000839*t2+0.0000050*t3;
       *z=0.6406161*t+0.0003041*t2+0.0000051*t3;
   *taeta=0.5567530*t-0.0001185*t2+0.0000116*t3;

} /* param */


void sin_cos(sx,cx,sz,cz,st,ct,zaeta,z,taeta)
double *sx,*cx,*sz,*cz,*st,*ct;
double zaeta,z,taeta;
{
   *sx=sin(rad(zaeta)); *cx=cos(rad(zaeta));
   *sz=sin(rad(z));     *cz=cos(rad(z));
   *st=sin(rad(taeta)); *ct=cos(rad(taeta));
} /* sin_cos */


void precession(alfa2,delta2,year2,month2,day2,alfa1,delta1,year1,month1,day1)
double *alfa2,*delta2;
int year2,month2,day2;
double alfa1,delta1;
int year1,month1,day1;

{
   double zaeta,taeta,z,jd1,jd2,cx,sx,cz,sz,ct,st;

   double p1[5][5];
   double p2[5][5];

   double v[5];
   double w[5];
   double s[5];


   jd1=julian_day(year1,month1,day1);
   param(&zaeta,&z,&taeta,jd1);
   sin_cos(&sx,&cx,&sz,&cz,&st,&ct,zaeta,z,taeta);

   p1[1][1]=cx*ct*cz-sx*sz;  p1[1][2]=cx*ct*sz+sx*cz;  p1[1][3]=cx*st;
   p1[2][1]=-sx*ct*cz-cx*sz; p1[2][2]=-sx*ct*sz+cx*cz; p1[2][3]=-sx*st;
   p1[3][1]=-st*cz;          p1[3][2]=-st*sz;          p1[3][3]=ct;

   v[1]=cos(alfa1)*cos(delta1);
   v[2]=sin(alfa1)*cos(delta1);
   v[3]=sin(delta1);

   matrix_mult(s,v,p1);

   jd2=julian_day(year2,month2,day2);
   param(&zaeta,&z,&taeta,jd2);
   sin_cos(&sx,&cx,&sz,&cz,&st,&ct,zaeta,z,taeta);

   p2[1][1]=cx*ct*cz-sx*sz; p2[1][2]=-sx*ct*cz-cx*sz; p2[1][3]=-st*cz;
   p2[2][1]=cx*ct*sz+sx*cz; p2[2][2]=-sx*ct*sz+cx*cz; p2[2][3]=-st*sz;
   p2[3][1]=cx*st;          p2[3][2]=-sx*st;          p2[3][3]=ct;

   matrix_mult(w,s,p2);

   *alfa2=atan2(w[2],w[1]);

   *delta2=asin(w[3]);

   if (*alfa2 < 0.0) {
      *alfa2=*alfa2+2.0*pi;
   } /* if */

} /* precession */ 


void equ_hor(altitude,azimuth,delta,fi,h)
double *altitude,*azimuth;
double delta,fi,h;
{
   double sd,sf,cf,sa,az;

   sd=sin(delta);
   sf=sin(fi); cf=cos(fi);
   sa=sd*sf+cos(delta)*cf*cos(h);
   *altitude=asin(sa);
   az=acos((sd-sf*sa)/(cf*cos(*altitude)));
   if (sin(h) > 0.0) {
      *azimuth=2.0*pi-az;
   }
   else {
      *azimuth=az;
   } /* if */

} /* equ_hor */ 


int main(argc,argv) 
int argc;
char *argv[];
{

   int year,month,day,hour,min,ha,ma,dd,md,epoch;

   int dla,mla,dlo,mlo;

   int degi,mini;
  
   double secr,zz;

   double sec,sa,sd,sla,slo;
   
   double alt,az,gst,alpha,delta,h,la,lo;

   double a1,a2,d1,d2;

   clock_t time1,timediff1;

   if (argc != 20) {
      printf("Usage: %s halphai malphai salphai\n",argv[0]);
      printf("              ddeltai mdeltai sdeltar\n");
      printf("              dloi    mloi    slor\n");
      printf("              dlai    mlai    slar\n");
      printf("              epochi\n");
      printf("              yeari monthi dayi houri mini secr \n");
      exit(0);
   } /* if */


   ha=atoi(argv[1]);
   ma=atoi(argv[2]);
   sa=atof(argv[3]);

   dd=atoi(argv[4]);
   md=atoi(argv[5]);
   sd=atof(argv[6]);

   dlo=atoi(argv[7]);
   mlo=atoi(argv[8]);
   slo=atof(argv[9]);

   dla=atoi(argv[10]);
   mla=atoi(argv[11]);
   sla=atof(argv[12]);

   epoch=atoi(argv[13]);

   year=atoi(argv[14]);
   month=atoi(argv[15]);
   day=atoi(argv[16]);

   hour=atoi(argv[17]);
   min=atoi(argv[18]);
   sec=atof(argv[19]);


   time1=clock();

   gst=ut_gst(year,month,day,dms_deg(hour,min,sec));

   alpha=rad(15.0*dms_deg(ha,ma,sa));
   delta=rad(dms_deg(dd,md,sd));
   lo=rad(dms_deg(dlo,mlo,slo));
   la=rad(dms_deg(dla,mla,sla));


   if (epoch > 0) {

      a1=alpha;
      d1=delta;

      precession(&a2,&d2,year,month,day,a1,d1,epoch,1,1);

      alpha=a2;
      delta=d2;


   } /* if */


   h=in_2pi(rad(15.0*gst)-alpha+lo);

   equ_hor(&alt,&az,delta,la,h);

   alt=deg(alt);
   az=deg(az);
   zz=90.0-alt;

   timediff1=clock()-time1;

   printf("\n");

   deg_dms(alt,&degi,&mini,&secr);
   printf("Altitude        = %12.8f deg = ",alt);   
   printf("%4d deg %2d min %5.2f sec\n",degi,mini,secr);

   printf("\n");

   deg_dms(az,&degi,&mini,&secr);
   printf("Azimuth         = %12.8f deg = ",az);   
   printf("%4d deg %2d min %5.2f sec\n",degi,mini,secr);

   printf("\n");

   deg_dms(zz,&degi,&mini,&secr);
   printf("Zenith distance = %12.8f deg = ",zz);   
   printf("%4d deg %2d min %5.2f sec\n",degi,mini,secr);


   printf("\n");

   /* printf("%8.2f\n",(double)(timediff1)/timetic); */

 
} /* End  */
