/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/***********************************************************************************/
/*                                                                                 */
/* Program:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* sigtest.c                                                                       */
/*                                                                                 */
/*                                                                                 */
/* Version:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* First version: December 1997                                                    */
/* Latest update: 2003-02-15                                                       */
/*                                                                                 */
/*                                                                                 */
/* Author:                                                                         */
/* =======                                                                         */
/*                                                                                 */
/* Program by: Stefan Spaennare                                                    */
/*                                                                                 */
/*                                                                                 */
/* Purpose:                                                                        */
/* ========                                                                        */
/*                                                                                 */
/* The program produces statistical data to test the program sigcol.c or other     */
/* similar programs. The program produce a text (ascii) table with 5 columns,      */
/* 3 with only text (letters) and 2 with Normal (Gaussian) and rectangular         */
/* distributed random numbers respectively. Normal random numbers are present      */
/* in column 2 and rectangular random numbers in column 4.                         */
/*                                                                                 */
/*                                                                                 */
/* Algorithms:                                                                     */
/* ===========                                                                     */
/*                                                                                 */
/* Algorithms for generating random numbers, see the references.                   */
/*                                                                                 */
/*                                                                                 */
/* References:                                                                     */
/* ===========                                                                     */
/*                                                                                 */
/* The book "Numerical Recipes in C", Second Edition, by W. H. Press at. al.,      */
/* 1992.                                                                           */
/*                                                                                 */
/*                                                                                 */
/* Compiling:                                                                      */
/* ==========                                                                      */
/*                                                                                 */
/* >gcc -O3 -o sigtest sigtest.c -lm                                               */
/*                                                                                 */
/*                                                                                 */
/* Input data:                                                                     */
/* ===========                                                                     */
/*                                                                                 */
/* >Usage: sigtest outfile meanr sigmar ni seedi                                   */
/*                                                                                 */
/* outfile: Output text (ascii) file with 5 columns.                               */
/* meanr: Mean value of random numbers in column 2 and 4.                          */
/* sigmar: Standard deviation of random numbers in column 2 and 4.                 */
/* ni: Number of rows in the table.                                                */
/* seedi: Random number seed (>=0).                                                */
/*                                                                                 */
/* Input variables ending with "i" are integers and ending "r" are real.           */
/*                                                                                 */
/*                                                                                 */
/***********************************************************************************/     


#include <math.h>
#include <stdio.h> 
#include <stdlib.h>
#include <unistd.h>
#include  <time.h>


double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


double gasdev(idum)
long *idum;
{
   static int iset=0;
   static double gset;
   double fac,rsq,v1,v2;

   if (iset == 0) {
      do {
         v1=2.0*ran2(idum)-1.0;
         v2=2.0*ran2(idum)-1.0;
         rsq=v1*v1+v2*v2;
      } while (rsq >= 1.0 || rsq == 0.0);
      fac=sqrt(-2.0*log(rsq)/rsq);
      iset=1;
      gset=v1*fac;
      return(v2*fac);
   } else {
      iset=0;
      return(gset);
   } /* if */

} /* gasdev */


int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *outfile;

   int i,n,seed;

   long idum;

   double mean,sigma,temp,temp2,sqrt12;

   clock_t time1,timediff1;

   if (argc != 6) {
      printf("Usage: %s outfile meanr sigmar ni seedi\n",argv[0]);
      exit(0);
   } /* if */
   
   outfile=fopen(argv[1],"w");

   mean=atof(argv[2]);
   sigma=atof(argv[3]);
   n=atoi(argv[4]);
   seed=atoi(argv[5]);


   idum=-abs(seed)-1;

   sqrt12=sqrt(12.0);

   time1=clock();

   for (i=1; i<=n; i++) {

       temp=mean+sigma*gasdev(&idum);

       temp2=mean+sqrt12*sigma*(ran2(&idum)-0.5);
       
       fprintf(outfile,"XXX %12.6f YYY %12.6f ZZZ\n",temp,temp2);

   } /* for i */  

   timediff1=clock()-time1;

   printf("%8.2f\n",(double)(timediff1)/timetic);

   fclose(outfile);

} /* End */
